/**
 * Relative Accuracy Measures for Stroke Gestures (.NET Framework 4.5 C# version)
 *
 * 	    Radu-Daniel Vatavu, Ph.D.
 *	    University Stefan cel Mare of Suceava
 *	    Suceava 720229, Romania
 *	    vatavu@eed.usv.ro
 *
 * 	    Lisa Anthony, Ph.D.
 *      UMBC
 *      Information Systems Department
 *      1000 Hilltop Circle
 *      Baltimore, MD 21250
 *      lanthony@umbc.edu
 *	
 *	    Jacob O. Wobbrock, Ph.D.
 * 	    The Information School
 *	    University of Washington
 *	    Seattle, WA 98195-2840
 *	    wobbrock@uw.edu
 *
 * The academic publication for the relative accuracy measures, and what should be 
 * used to cite it, is:
 *
 *	Vatavu, R.-D., Anthony, L. and Wobbrock, J.O. (2013).  
 *	  Relative accuracy measures for stroke gestures. 
 *	  In Proceedings of the 15th ACM on International Conference 
 *	  on Multimodal Interaction (ICMI '13). ACM, New York, NY, USA, 279-286
 *	  DOI=10.1145/2522848.2522875 http://doi.acm.org/10.1145/2522848.2522875
 *
 * This software is distributed under the "New BSD License" agreement:
 *
 * Copyright (c) 2013, Radu-Daniel Vatavu, Lisa Anthony, and 
 * Jacob O. Wobbrock. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *    * Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *    * Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *    * Neither the names of the University Stefan cel Mare of Suceava, 
 *	    University of Washington, nor UMBC, nor the names of its contributors 
 *	    may be used to endorse or promote products derived from this software 
 *	    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Radu-Daniel Vatavu OR Lisa Anthony
 * OR Jacob O. Wobbrock BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
**/
using System;

namespace RelativeAccuracyMeasures
{
	public class Measurements
    {
        /// <summary>
        /// Returns the squared Euclidean Distance between two points.
        /// </summary>
        public static double SqrEuclideanDistance(Point a, Point b)
        {
            return (a.X - b.X) * (a.X - b.X) + (a.Y - b.Y) * (a.Y - b.Y);
        }

        /// <summary>
        /// Returns the Euclidean Distance between two points.
        /// </summary>
        public static double EuclideanDistance(Point a, Point b)
        {
            return Math.Sqrt(SqrEuclideanDistance(a, b));
        }

        /// <summary>
        /// Returns the smallest turning angle between vectors v and u in radians in [0..PI].
		/// </summary>
        public static double ShortAngle(Vector v, Vector u)
		{
            // compute cosine between vectors
            double vLength = v.Length();
            double uLength = u.Length();
            if (Math.Abs(vLength * uLength) <= Double.Epsilon)
                return 0;
            double cos_angle = Vector.DotProduct(v, u) / (vLength * uLength);
            
            // deal with special cases
            if (cos_angle <= -1.0) return Math.PI;
			if (cos_angle >= 1.0) return 0;

            // return angle in the interval [0, PI]
            return Math.Acos(cos_angle);
		}

		/// <summary>
        /// Returns the trigonometric turning angle between vectors v and u in radians [0..2PI]
        /// </summary>
        public static double Angle(Vector v, Vector u)
		{
            double angle = ShortAngle(v, u);
            if (!Measurements.TrigonometricOrder(v, u))
				angle = Math.PI * 2.0 - angle;
			return angle;
		}
    
        /// <summary>
        /// Returns true if vectors v and u are in trigonometric order
        /// </summary>
        public static bool TrigonometricOrder(Vector v, Vector u)
        {
            return (v.Values.X * u.Values.Y - v.Values.Y * u.Values.X < 0) ? false : true;
        }
    }
}