/**
 * Relative Accuracy Measures for Stroke Gestures (.NET Framework 4.5 C# version)
 *
 * 	    Radu-Daniel Vatavu, Ph.D.
 *	    University Stefan cel Mare of Suceava
 *	    Suceava 720229, Romania
 *	    vatavu@eed.usv.ro
 *
 * 	    Lisa Anthony, Ph.D.
 *      UMBC
 *      Information Systems Department
 *      1000 Hilltop Circle
 *      Baltimore, MD 21250
 *      lanthony@umbc.edu
 *	
 *	    Jacob O. Wobbrock, Ph.D.
 * 	    The Information School
 *	    University of Washington
 *	    Seattle, WA 98195-2840
 *	    wobbrock@uw.edu
 *
 * The academic publication for the relative accuracy measures, and what should be 
 * used to cite it, is:
 *
 *	Vatavu, R.-D., Anthony, L. and Wobbrock, J.O. (2013).  
 *	  Relative accuracy measures for stroke gestures. 
 *	  In Proceedings of the 15th ACM on International Conference 
 *	  on Multimodal Interaction (ICMI '13). ACM, New York, NY, USA, 279-286
 *	  DOI=10.1145/2522848.2522875 http://doi.acm.org/10.1145/2522848.2522875
 *
 * This software is distributed under the "New BSD License" agreement:
 *
 * Copyright (c) 2013, Radu-Daniel Vatavu, Lisa Anthony, and 
 * Jacob O. Wobbrock. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *    * Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *    * Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *    * Neither the names of the University Stefan cel Mare of Suceava, 
 *	    University of Washington, nor UMBC, nor the names of its contributors 
 *	    may be used to endorse or promote products derived from this software 
 *	    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Radu-Daniel Vatavu OR Lisa Anthony
 * OR Jacob O. Wobbrock BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
**/
using System;

namespace RelativeAccuracyMeasures
{
    /// <summary>
    /// 
    /// Implements a point with (X, Y) coordinates, timestamp (T), and stroke identifier (StrokeId).
    /// 
    /// </summary>
	public class Point
	{
		public double X, Y;         // 2-D coordinates
        public double T;            // time
        public int StrokeID;        // the ID of the stroke (e.g., 0, 1, 2, ...) 

		/// <summary>
		/// Creates a point with (0, 0, 0, 0) coordinates
		/// </summary>
		public Point()
		{	
			this.LoadMemberData(0, 0, 0, 0);
		}

        /// <summary>
        /// Creates a point with (x, y, t, strokeID) coordinates
        /// </summary>
        public Point(double x, double y, double t, int strokeID)
        {
            this.LoadMemberData(x, y, t, strokeID);
        }

        /// <summary>
		/// Creates a point identical with the argument point
        /// (copy constructor)
		/// </summary>
		public Point(Point p)
		{
			this.LoadMemberData(p.X, p.Y, p.T, p.StrokeID);
		}
		
	    /// <summary>
        /// Implements a private instance constructor
        /// </summary>
        private void LoadMemberData(double x, double y, double t, int strokeID)
		{
			X = x;
			Y = y;
			T = t;
            StrokeID = strokeID;
		}

		/// <summary>
        /// Returns the point with minimum coordinates on X and Y axes
        /// </summary>
        public static Point AbsoluteMin
        {
            get
            {
                return new Point(double.MinValue, double.MinValue, 0, 0);
            }
        }

        /// <summary>
        /// Returns the point with maximum coordinates on X and Y axes
        /// </summary>
        public static Point AbsoluteMax
        {
            get
            {
                return new Point(double.MaxValue, double.MaxValue, 0, 0);
            }
        }

		/// <summary>
		/// Returns the max value of the X and Y coordinates of this point
		/// </summary>
        public double MaxXY
		{
			get
			{
				return Math.Max(X, Y);
			}
		}

		/// <summary>
		/// Returns p1 + p2 on each coordinate and copies the timestamp T and StrokeID of p1
		/// </summary>
		public static Point operator +(Point p1, Point p2)
		{
			return new Point(p1.X + p2.X, p1.Y + p2.Y, p1.T, p1.StrokeID);
		}

		/// <summary>
        /// Returns p1 - p2 on each coordinate and copies the timestamp T and StrokeID of p1
		/// </summary>
		public static Point operator -(Point p1, Point p2)
		{
			return new Point(p1.X - p2.X, p1.Y - p2.Y, p1.T, p1.StrokeID);
		}

		/// <summary>
        /// Returns p / scalar on each coordinate.
        /// Throws DivideByZeroException if abs(scalar) is less than 10e-6
		/// </summary>
        public static Point operator /(Point p, double scalar)
		{
            if (Math.Abs(scalar) < 10e-6)
                throw new DivideByZeroException(String.Format("Scalar is less than 10e-6 [{0}].", scalar));
			return new Point(p.X / scalar, p.Y / scalar, p.T, p.StrokeID);
		}

		/// <summary>
        /// Returns scalar * p on each coordinate
		/// </summary>
        public static Point operator *(double scalar, Point p)
		{
			return new Point(scalar * p.X, scalar * p.Y,p.T, p.StrokeID);
		}
	}
}
