/**
 * Relative Accuracy Measures for Stroke Gestures (.NET Framework 4.5 C# version)
 *
 * 	    Radu-Daniel Vatavu, Ph.D.
 *	    University Stefan cel Mare of Suceava
 *	    Suceava 720229, Romania
 *	    vatavu@eed.usv.ro
 *
 * 	    Lisa Anthony, Ph.D.
 *      UMBC
 *      Information Systems Department
 *      1000 Hilltop Circle
 *      Baltimore, MD 21250
 *      lanthony@umbc.edu
 *	
 *	    Jacob O. Wobbrock, Ph.D.
 * 	    The Information School
 *	    University of Washington
 *	    Seattle, WA 98195-2840
 *	    wobbrock@uw.edu
 *
 * The academic publication for the relative accuracy measures, and what should be 
 * used to cite it, is:
 *
 *	Vatavu, R.-D., Anthony, L. and Wobbrock, J.O. (2013).  
 *	  Relative accuracy measures for stroke gestures. 
 *	  In Proceedings of the 15th ACM on International Conference 
 *	  on Multimodal Interaction (ICMI '13). ACM, New York, NY, USA, 279-286
 *	  DOI=10.1145/2522848.2522875 http://doi.acm.org/10.1145/2522848.2522875
 *
 * This software is distributed under the "New BSD License" agreement:
 *
 * Copyright (c) 2013, Radu-Daniel Vatavu, Lisa Anthony, and 
 * Jacob O. Wobbrock. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *    * Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *    * Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *    * Neither the names of the University Stefan cel Mare of Suceava, 
 *	    University of Washington, nor UMBC, nor the names of its contributors 
 *	    may be used to endorse or promote products derived from this software 
 *	    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Radu-Daniel Vatavu OR Lisa Anthony
 * OR Jacob O. Wobbrock BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
**/
using System;

namespace RelativeAccuracyMeasures
{
    /// <summary>
    /// 
    /// Contains utility functions for point sets.
    /// 
    /// </summary>
	public static class PointSet
	{
        #region centroid, min, max, bounding box, path length

        /// <summary>
		/// Returns the center of gravity for a set of points
        /// Complexity: O(n)
		/// </summary>
        public static Point Centroid(Point[] points)
        {
            if (points == null || points.Length == 0)
                return new Point();

            Point cg = new Point();
            foreach(Point point in points)
                cg += point;
            cg /= points.Length;
            return cg;
        }

		/// <summary>
		/// Returns a point that has minimum values on the X and Y axes.
		/// Complexity: O(n)
		/// </summary>
        public static Point Min(Point[] points)
        {
            if (points == null || points.Length == 0)
                return new Point();

            Point min = Point.AbsoluteMax;
            foreach(Point point in points)
            {
                if (min.X > point.X) min.X = point.X;
                if (min.Y > point.Y) min.Y = point.Y;
            }
            return min;
        }

		/// <summary>
        /// Returns a point that has maximum values on the X and Y axes.
        /// Complexity: O(n)
		/// </summary>
        public static Point Max(Point[] points)
        {
            if (points == null || points.Length == 0)
                return new Point();

            Point max = Point.AbsoluteMin;
            foreach(Point point in points)
            {
                if (max.X < point.X) max.X = point.X;
                if (max.Y < point.Y) max.Y = point.Y;
            }
            return max;
        }

        /// <summary>
        /// Returns the bounding box of the set of points.
        /// Complexity: O(n)
        /// </summary>
        public static Rectangle BoundingBox(Point[] points)
        {
            return new Rectangle(Min(points), Max(points));
        }

		/// <summary>
        /// Computes path length.
        /// Complexity: O(n)
        /// </summary>
        public static double PathLength(Point[] points)
        {
            return PathLength(points, 0, points.Length - 1);
        }

        /// <summary>
        /// Computes path length.
        /// Complexity: O(n)
        /// </summary>
        public static double PathLength(Point[] points, int p, int q)
        {
            if (points == null || points.Length == 0 || p >= q)
                return 0;

            double length = 0;
            for (int i = p + 1; i <= q; i++)
                if (points[i].StrokeID == points[i - 1].StrokeID)
                    length += Measurements.EuclideanDistance(points[i], points[i - 1]);
            return length;
        }

        #endregion

        #region scale to, translate by, resample in space

        /// <summary>
        /// Scales a set of points with shape preservation.
        /// Complexity: O(n)
        /// </summary>
        public static Point[] ScaleTo(Point[] points)
        {
            Point min = PointSet.Min(points);
            Point max = PointSet.Max(points);
            double scaleFactor = 1.0 / (max - min).MaxXY;

            // scale points
            Point[] newPoints = new Point[points.Length];
            if (!double.IsInfinity(scaleFactor) && Math.Abs(scaleFactor) > 1e-10)
            {
                for (int i = 0; i < points.Length; i++)
                    newPoints[i] = scaleFactor * (points[i] - min);
            }
            else
            {
                for (int i = 0; i < points.Length; i++)
                    newPoints[i] = new Point(points[i]);
            }
            return newPoints;
        }

        /// <summary>
        /// Translate points by offset. 
        /// If offset coincides with the center of gravity, the new center will be (0, 0).
        /// Complexity: O(n)
        /// </summary>
        public static Point[] TranslateBy(Point[] points, Point offset)
        {
            Point[] newPoints = new Point[points.Length];
            for (int i = 0; i < newPoints.Length; i++)
                newPoints[i] = points[i] - offset;
            return newPoints;
        }

        /// <summary>
        /// Resample the point set uniformly into a custom number of points n.
        /// Complexity: O(n)
        /// </summary>
        public static Point[] Resample(Point[] points, int n)
        {
            if (points.Length == 0 || n <= 0)
                return new Point[0];

            Point[] newPoints = new Point[n];
            newPoints[0] = new Point(points[0]);
            if (points.Length == 1)
            {
                for (int i = 1; i < newPoints.Length; i++)
                    newPoints[i] = new Point(points[0]);
                return newPoints;
            }

            int numPoints = 1;
            double I = PathLength(points) / (n - 1); // computes interval length
            double D = 0;
            for (int i = 1; i < points.Length; i++)
                if (points[i].StrokeID == points[i - 1].StrokeID)
                {
                    double d = Measurements.EuclideanDistance(points[i - 1], points[i]);
                    if (D + d >= I)
                    {
                        Point firstPoint = points[i - 1];
                        while (D + d >= I)
                        {
                            // add interpolated point
                            double t = Math.Min(Math.Max((I - D) / d, 0.0), 1.0);
                            if (double.IsNaN(t)) t = 0.5;
                            newPoints[numPoints++] = new Point(
                                (1.0 - t) * firstPoint.X + t * points[i].X,
                                (1.0 - t) * firstPoint.Y + t * points[i].Y,
                                (1.0 - t) * firstPoint.T + t * points[i].T,
                                points[i].StrokeID
                            );

                            // update partial length
                            d = D + d - I;
                            D = 0;
                            firstPoint = newPoints[numPoints - 1];
                        }
                        D = d;
                    }
                    else D += d;
                }

            if (numPoints == n - 1) // sometimes we fall a rounding-error short of adding the last point, so add it if so
                newPoints[numPoints++] = new Point(points[points.Length - 1]);
            return newPoints;
        }
        
        #endregion
    }
}
